<?php
namespace Pinpoint\BlankProduct\Plugin\Model\Stock;

use \Magento\Framework\Registry;
use \Magento\Backend\Model\Auth\Session as AuthSession;

/**
 * Class ItemPlugin
 * @package Pinpoint\BlankProduct\Plugin\Model\Stock
 */
class ItemPlugin
{
    /**
     * @var Registry
     */
    protected $_coreRegistry;

    /**
     * @var AuthSession
     */
    protected $_authSession;

    /**
     * @var \Pinpoint\BlankProduct\Helper\Quantity
     */
    protected $quantityHelper;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * ItemPlugin constructor.
     * @param Registry $coreRegistry
     * @param AuthSession $authSession
     * @param \Pinpoint\BlankProduct\Helper\Quantity $quantityHelper
     * @param \Magento\Framework\App\Request\Http $request
     */
    public function __construct(
        Registry $coreRegistry,
        AuthSession $authSession,
        \Pinpoint\BlankProduct\Helper\Quantity $quantityHelper,
        \Magento\Framework\App\Request\Http $request
    ) {
        $this->_coreRegistry = $coreRegistry;
        $this->_authSession = $authSession;
        $this->quantityHelper = $quantityHelper;
        $this->request = $request;
    }

    /**
     * @param \Magento\CatalogInventory\Model\Stock\Item $subject
     * @param $result
     * @return bool|int
     */
    public function afterGetIsInStock(\Magento\CatalogInventory\Model\Stock\Item $subject, $result)
    {
        if(
            !$this->_authSession->isLoggedIn() && !is_null($this->request->getControllerModule())
            || $this->_authSession->isLoggedIn() && $this->request->getControllerModule() != "Magento_Catalog" && $this->request->getActionName() != "save"
        ) {
            if ($this->_coreRegistry->registry('stockitem_getisinstock') !== 'value') {
                $this->_coreRegistry->register('stockitem_getisinstock', 'value');
                if ($result === false) {
                    // look up this product to see if there are any blank products
                    $blankProduct = $this->quantityHelper->getBlankProductStockItem($subject->getProductId());
                    if ($blankProduct) {
                        $result = $blankProduct;
                        $this->_coreRegistry->unregister('stockitem_getisinstock');
                        return $result;
                    }
                }
                $this->_coreRegistry->unregister('stockitem_getisinstock');
            }
        }
        return $result;
    }
}
